#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module contains a function to evaluate the saturation of an image.

Created on Wed Apr 10 11:49:45 2018
Last edited on Fri Aug 2 11:46:24 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

###############################################################################
#                                                                             #
#                                   Libraries                                 #
#                                                                             #
###############################################################################

import os # to handle filesystem files
import cv2 # for image manipulation
import numpy as np # numerical computation
import pandas as pd # (not used in this script, but imported)

###############################################################################
#                                                                             #
#                              Saturation                                     #
#                                                                             #
###############################################################################

""" This section handles the computation of the saturation of an image. """

def saturation(img):
    """ 
    This function evaluates the saturation of an image.
        
    The image is first converted to the HSV color space, then the mean saturation value is computed.
    
    :param img: Image to analyze, in RGB format.
    :type img: numpy.ndarray
    :return: Mean saturation value of the image.
    :rtype: float
    """
    
    img_hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV) / 255  # Convert image to HSV color space and normalize to [0, 1]
    saturation = img_hsv[:, :, 1].mean()  # Compute the mean of the Saturation channel
    
    return saturation  # Return the mean saturation value

###############################################################################
#                                                                             #
#                                  DEBUG                                      #
#                                                                             #
###############################################################################
        
""" For debug purposes: reads sample images and computes their saturation values. """

if __name__ == '__main__':

    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "panda.jpg"
    
    img = cv2.imread(sample_img)  # Read the image
    print(sample_img, saturation(img))  # Print the image path and its saturation value



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module is used to detect (frontal) faces within an image.
It uses OpenCV's (cv2) Haar cascade for the detection or the package face-detection.
CV2 model is faster but less accurate (good for front-facing images).
Detection using the face-detection package can be done via the 'hog' or 'cnn' methods.
See the face-detection package documentation for details.

Created on Mon Apr 16 22:40:46 2018
Last edited on Fri Aug 3 11:52:14 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

import os
import cv2
import matplotlib.pyplot as plt
import numpy as np
import face_recognition
from PIL import Image

###############################################################################
#                                                                             #
#                      Frontal Faces Detection                                #
#                                                                             #
###############################################################################

def detect_faces_cv2(img, plot=False):
    """ This function uses CV2 to detect faces in a picture.
    
        :param img: image to analyze in RGB
        :type img: numpy.ndarray
        :param plot: whether to plot or not the results
        :type plot: bool
        :return: list of detected faces as rectangles
        :rtype: list
    """
    # Convert the image to grayscale as Haar cascades work better on grayscale images
    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    
    # Load the Haar cascade for frontal face detection
    frontalface_cascade = cv2.CascadeClassifier(cv2.data.haarcascades + 'haarcascade_frontalface_default.xml')
    
    # Detect faces in the image
    faces = frontalface_cascade.detectMultiScale(img, 1.3, 5)
    
    # If plot is True, draw rectangles around detected faces and display the image
    if plot:
        for (x, y, w, h) in faces:
            cv2.rectangle(img, (x, y), (x + w, y + h), (255, 0, 0), 2)
            plt.imshow(img)

    # Return the list of detected faces
    return faces

def detect_faces(imgpath, plot=False, model='hog'):
    """ This function uses face-recognition to detect faces in a picture.
        By default it uses the hog method. cnn method can be passed as model parameter.
    
        :param img: path to the image to analyze
        :type img: string
        :param plot: whether to plot or not the results
        :type plot: bool
        :param model: which model to use for the detection of faces (hog or cnn). Default is 'hog'.
        :type model: string
        :return: list of detected faces as rectangles
        :rtype: list
    """

    # Convert the image to grayscale as Haar cascades work better on grayscale images
    image = face_recognition.load_image_file(imgpath)
    faces = face_recognition.face_locations(image, model=model)
    # If plot is True, draw rectangles around detected faces and display the image
    if plot:
        for top, right, bottom, left in faces:
            cv2.rectangle(image, (left, top), (right, bottom), (255, 0, 0), 2)
            plt.imshow(image)

    # if plot:
    #     for top, right, bottom, left in faces:
    #     # You can access the actual face itself like this:
    #         face_image = image[top:bottom, left:right]
    #         pil_image = Image.fromarray(face_image)
    #         pil_image.show()
        
    # Return the list of detected faces
    return faces

###############################################################################
#                                                                             #
#                                   DEBUG                                     #
#                                                                             #
###############################################################################

if __name__ == '__main__':
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "face1.png"
    
    # Read the sample image
    img = cv2.imread(sample_img)
    
    # Convert the image to RGB
    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    
    # Detect faces in the image and plot the results
    faces = detect_faces_cv2(img, plot=True)
    
    # Print the number of faces detected
    print("Number of faces in the picture is:", len(faces))

    # Detect faces in the image and plot the results
    faces = detect_faces(sample_img, plot=True)
    
    # Print the number of faces detected
    print("Number of faces in the picture is:", len(faces))

    


# -*- coding: utf-8 -*-

#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module contains functions to compute the number of independent areas in an image.

Created on Sat Apr 21 09:40:45 2018
Last edited on Fri Aug 2 12:49:23 2024

@author: giulio
"""

import os  # To handle filesystem files
import cv2  # For image manipulation
import numpy as np  # For numerical operations
from imutils import perspective  # For image perspective transformations
from imutils import contours  # For contour-related utilities
import imutils  # For various image utilities
from scipy import ndimage  # For scientific image processing
import matplotlib.pyplot as plt  # For data visualization
import matplotlib.patches as patches  # For legends and rectangle drawing
import pytesseract  # For OCR
from PIL import Image  # For image processing with PIL

try:
    from . import utils
except: 
    import utils

###############################################################################
#                                                                             #
#                               Text to Image Ratio                           #
#                                                                             #
###############################################################################

def text2image_ratio(areas):
    """ This function evaluates the text-to-image ratio, as well as the total area occupied by both image and text.
    
        :param areas: areas dict as extracted by the get_areas function
        :type areas: dict
        :return: a dict containing the text / (image+text) ratio, total area of text, total area of images, and number of images
        :rtype: dict
    """
    
    image = []
    text = []
    for area in areas:
        if areas[area]["type"] == "Text":
            text.append(areas[area]["area"])
        else:
            image.append(areas[area]["area"])

    # Calculate ratio of text to the total area (text + image)
    if (sum(image) + sum(text)) > 0:
        ratio = sum(text) / (sum(image) + sum(text))
    else:
        ratio = np.nan
    return {"text2image_ratio": ratio, "textArea": sum(text), "imageArea": sum(image), "nImages": len(image)}

###############################################################################
#                                                                             #
#                                Get Areas                                    #
#                                                                             #
###############################################################################

def get_areas(img, minArea=100, resize=True, newSize=[600, 400], plot=False, coordinates=False, areatype=True, returnbox=False):
    """ Detects and returns areas in an image based on contours.
    
    :param img: Source image.
    :type img: numpy.ndarray
    :param minArea: Minimum area threshold for detected objects.
    :type minArea: int
    :param resize: Whether to resize the image.
    :type resize: bool
    :param newSize: New size for resizing the image.
    :type newSize: list
    :param plot: Whether to plot the detected areas.
    :type plot: bool
    :param coordinates: Whether to include coordinates in the result.
    :type coordinates: bool
    :param areatype: Whether to classify areas as text or image.
    :type areatype: bool
    :param returnbox: Whether to include box points in the result.
    :type returnbox: bool
    :return: Dictionary with detected areas and their properties.
    :rtype: dict
    """
    
    img_original = img  # Copy the original image
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)  # Convert to grayscale
    gray = cv2.GaussianBlur(gray, (7, 7), 0)  # Apply Gaussian blur
    # Perform edge detection, then perform dilation + erosion to close gaps in between object edges
    edged = cv2.Canny(gray, 50, 100)  # Detect edges
    edged = cv2.dilate(edged, None, iterations=1)  # Dilate edges
    edged = cv2.erode(edged, None, iterations=1)  # Erode edges to refine
    # Find contours in the edge map
    cnts = cv2.findContours(edged.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
    cnts = imutils.grab_contours(cnts)  # Grab contours
    
    areas = {}
    
    if len(cnts) > 0:
        # Sort the contours from left-to-right
        (cnts, _) = contours.sort_contours(cnts)
        
        boxes = []
        for c in cnts:
            # Compute bounding box for each contour
            x, y, w, h = cv2.boundingRect(c)
            box = [x, y, x + w, y + h]
            
            minX, minY, maxX, maxY = box
            area = (maxX - minX) * (maxY - minY)
            if area > minArea:
                boxes.append(box)
        
        if plot:
            # Plot the detected bounding boxes
            img_plot = img_original.copy()
            for box in boxes:
                cv2.rectangle(img_plot, (box[0], box[1]), (box[2], box[3]), (0, 255, 0), 2)
            plt.figure("Space-based Decomposition")
            plt.imshow(cv2.cvtColor(img_plot, cv2.COLOR_BGR2RGB))
            plt.title("Space-based Decomposition")
            plt.xticks([])
            plt.yticks([])
            plt.show()
        
        for i, box in enumerate(boxes):
            minX, minY, maxX, maxY = box
            imgportion = img_original[minY:maxY, minX:maxX]
            area = (maxX - minX) * (maxY - minY)
            
            if len(imgportion) > 0:
                if areatype:
                    if utils.textdetection(imgportion) > 0:
                        areas[i] = {"area": area, "type": "Text"}
                    else:
                        areas[i] = {"area": area, "type": "Image"}
                else:
                    areas[i] = {"area": area}
                if coordinates:
                    areas[i]['coordinates'] = {"xmin": minX, "xmax": maxX, "ymin": minY, "ymax": maxY}
                if returnbox:
                    areas[i]['box'] = box
                        
    return areas

###############################################################################
#                                                                             #
#                                  DEBUG                                      #
#                                                                             #
###############################################################################
        
""" For debug purposes."""

import math

if(__name__=='__main__'):

    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "panda.jpg"
    
    # Read the image with alpha channel
    img = cv2.imread(sample_img, cv2.IMREAD_UNCHANGED)
    # Convert BGRA image to RGBA
    img = cv2.cvtColor(img, cv2.COLOR_BGRA2RGBA)
    
    # Detect areas in the image
    areas = get_areas(img, minArea=10, plot=True, coordinates=True, areatype=True, resize=True, returnbox=True)
  


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module contains functions to evaluate the presence of different colors in an image.
It uses the 16 basic colors or the full 140 color list defined in the W3C specifications.

Created on Thu Apr 19 22:40:46 2018
Last edited on Fri Aug 2 11:37:57 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

import os
import cv2
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.patches as patches
import time
import pandas as pd
###############################################################################
#                                                                             #
#                  This section handles color recognition                     #
#                                                                             #
###############################################################################

# Define color dictionaries for 16 and 140 W3C colors
_colors = {
    16: {
        "Aqua": [0, 255, 255],
        "Black": [0, 0, 0],
        "Blue": [0, 0, 255],
        "Fuchsia": [255, 0, 255],
        "Gray": [128, 128, 128],
        "Green": [0, 128, 0],
        "Lime": [0, 255, 0],
        "Maroon": [128, 0, 0],
        "Navy": [0, 0, 128],
        "Olive": [128, 128, 0],
        "Purple": [128, 0, 128],
        "Red": [255, 0, 0],
        "Silver": [192, 192, 192],
        "Teal": [0, 128, 128],
        "White": [255, 255, 255],
        "Yellow": [255, 255, 0]
    },
    140: {
        "AliceBlue" : [240,248,255],
        "AntiqueWhite" : [250,235,215],
        "Aqua" : [0,255,255],
        "AquaMarine" : [127,255,212],
        "Azure" : [240,255,255],
        "Beige" : [245,245,220],
        "Bisque" : [255,228,196],
        "Black" : [0,0,0],
        "BlanchedAlmond" : [255,235,205],
        "Blue" : [0,0,255],
        "BlueViolet" : [138,43,226],
        "Brown" : [165,42,42],
        "BurlyWood" : [222,184,135],
        "CadetBlue" : [95,158,160],
        "Chartreuse" : [127,255,0],
        "Chocolate" : [210,105,30],
        "Coral" : [255,127,80],
        "CornFlowerBlue" : [100,149,237],
        "Cornsilk" : [255,248,220],
        "Crimson" : [220,20,60],
        "Cyan" : [0,255,255],
        "DarkBlue" : [0,0,139],
        "DarkCyan" : [0,139,139],
        "DarkGoldenRod" : [184,134,11],
        "DarkGray" : [169,169,169],
        "DarkGreen" : [0,100,0],
        "DarkKhaki" : [189,183,107],
        "DarkMagenta" : [139,0,139],
        "DarkOliveGreen" : [85,107,47],
        "DarkOrange" : [255,140,0],
        "DarkOrchid" : [153,50,204],
        "DarkRed" : [139,0,0],
        "DarkSalmon" : [233,150,122],
        "DarkSeaGreen" : [143,188,143],
        "DarkSlateBlue" : [72,61,139],
        "DarkSlateGray" : [47,79,79],
        "DarkTurquoise" : [0,206,209],
        "DarkViolet" : [148,0,211],
        "DeepPink" : [255,20,147],
        "DeepSkyBlue" : [0,191,255],
        "DimGray" : [105,105,105],
        "DodgerBlue" : [30,144,255],
        "FireBrick" : [178,34,34],
        "FloralWhite" : [255,250,240],
        "ForestGreen" : [34,139,34],
        "Fuchsia" : [255, 0, 255], 
        "Gainsboro" : [220,220,220],
        "GhostWhite" : [248,248,255],
        "Gold" : [255,215,0],
        "GoldenRod" : [218,165,32],
        "Gray" : [128,128,128],
        "Green" : [0,128,0],
        "GreenYellow" : [173,255,47],
        "HoneyDew" : [240,255,240],
        "HotPink" : [255,105,180],
        "IndianRed" : [205,92,92],
        "Indigo" : [75,0,130],
        "Ivory" : [255,255,240],
        "Khaki" : [240,230,140],
        "Lavender" : [230,230,250],
        "LavenderBlush" : [255,240,245],
        "LawnGreen" : [124,252,0],
        "LemonChiffon" : [255,250,205],
        "LightBlue" : [173,216,230],
        "LightCoral" : [240,128,128],
        "LightCyan" : [224,255,255],
        "LightGoldenrodYellow" : [250,250,210],
        "LightGray" : [211,211,211],
        "LightGreen" : [144,238,144],
        "LightPink" : [255,182,193],
        "LightSalmon" : [255,160,122],
        "LightSeaGreen" : [32,178,170],
        "LightSkyBlue" : [135,206,250],
        "LightSlateGray" : [119,136,153],
        "LightSteelBlue" : [176,196,222],
        "LightYellow" : [255,255,224],
        "Lime" : [0,255,0],
        "LimeGreen" : [50,205,50],
        "Linen" : [250,240,230],
        "Magenta" : [255,0,255],
        "Maroon" : [128,0,0],
        "MediumAquaMarine" : [102,205,170],
        "MediumBlue" : [0,0,205],
        "MediumOrchid" : [186,85,211],
        "MediumPurple" : [147,112,219],
        "MediumSeaGreen" : [60,179,113],
        "MediumSlateBlue" : [123,104,238],
        "MediumSpringGreen" : [0,250,154],
        "MediumTurquoise" : [72,209,204],
        "MediumVioletRed" : [199,21,133],
        "MidnightBlue" : [25,25,112],
        "MintCream" : [245,255,250],
        "MistyRose" : [255,228,225],
        "Moccasin" : [255,228,181],
        "NavajoWhite" : [255,222,173],
        "Navy" : [0,0,128],
        "OldLace" : [253,245,230],
        "Olive" : [128,128,0],
        "OliveDrab" : [107,142,35],
        "Orange" : [255,165,0],
        "OrangeRed" : [255,69,0],
        "Orchid" : [218,112,214],
        "PaleGoldenRod" : [238,232,170],
        "PaleGreen" : [152,251,152],
        "PaleTurquoise" : [175,238,238],
        "PaleVioletRed" : [219,112,147],
        "PapayaWhip" : [255,239,213],
        "PeachPuff" : [255,218,185],
        "Peru" : [205,133,63],
        "Pink" : [255,192,203],
        "Plum" : [221,160,221],
        "PowderBlue" : [176,224,230],
        "Purple" : [128,0,128],
        "Red" : [255,0,0],
        "RosyBrown" : [188,143,143],
        "RoyalBlue" : [65,105,225],
        "SaddleBrown" : [139,69,19],
        "Salmon" : [250,128,114],
        "SandyBrown" : [244,164,96],
        "SeaGreen" : [46,139,87],
        "SeaShell" : [255,245,238],
        "Sienna" : [160,82,45],
        "Silver" : [192,192,192],
        "SkyBlue" : [135,206,235],
        "SlateBlue" : [106,90,205],
        "SlateGray" : [112,128,144],
        "Snow" : [255,250,250],
        "SpringGreen" : [0,255,127],
        "SteelBlue" : [70,130,180],
        "Tan" : [210,180,140],
        "Teal" : [0,128,128],
        "Thistle" : [216,191,216],
        "Tomato" : [255,99,71],
        "Turquoise" : [64,224,208],
        "Violet" : [238,130,238],
        "Wheat" : [245,222,179],
        "White" : [255,255,255],
        "WhiteSmoke" : [245,245,245],
        "Yellow" : [255,255,0],
        "YellowGreen" : [154,205,50]
        }
    }

def get_colors_w3c(img, ncolors=16, plot=False, plotncolors=5):
    """
    This function is used to get a simplified color palette (W3C colors).
    It can be used with 16 (https://www.html-color-names.com/basic-color-names.php) 
    or 140 colors (https://www.w3schools.com/colors/colors_names.asp).
    
    :param img: Image to analyze in RGB format.
    :type img: numpy.ndarray
    :param ncolors: Number of colors to use (e.g., 16 or 140). Default is 16.
    :type ncolors: int
    :param plot: Whether to plot a color palette image showing the top colors.
    :type plot: bool
    :param plotncolors: Number of colors to display in the palette image. Default is 5.
    :type plotncolors: int
    :param clusterfactor: Factor to cluster similar colors. Default is 50.
    :type clusterfactor: int
    :return: An array of RGB values representing the most frequent colors.
    :rtype: numpy.ndarray
    """
    
    # Validate ncolors
    if ncolors not in [16, 140]:
        raise ValueError("Invalid value for 'ncolors'. Value must be 16 or 140.")
    
    # Select color dictionary
    colors = _colors[ncolors]
                 
    # Calculate the distance of each pixel to each color
    colors_array = np.array(list(colors.values()))
    dists = np.sum(np.abs(img[:, :, np.newaxis, :3] - colors_array), axis=3)
    closest_color_indices = np.argmin(dists, axis=2)
    
    colorscheme = []

    # Map the indices to color names
    for row_indices in closest_color_indices:
        row_colors = [list(colors.keys())[index] for index in row_indices]
        colorscheme.extend(row_colors)
    
    # Handle alpha channel
    if img.shape[2] == 4:
        alpha = img[:, :, 3]
        # Exclude completely transparent pixels (alpha == 0) from distance calculation
        mask = alpha > 100 
        mask = mask.ravel()
        colorscheme = np.array(colorscheme)[mask]
        
    # Calculate the percentage of each color
    unique_colors, counts = np.unique(colorscheme, return_counts=True)
    colorscheme = sorted([[c, count / len(colorscheme) * 100] for c, count in zip(unique_colors, counts)])
    
    # Add missing colors with 0% presence
    missingcolors = list(set(colors) - set(unique_colors))
    for color in missingcolors:
        colorscheme.append([color, 0.0])
    
    colorscheme = sorted(colorscheme)
    
    # Plot the color palette if required
    if plot:
        sorted_data = sorted(colorscheme, key=lambda x: x[1], reverse=True)
        fig, ax = plt.subplots()
        plt.suptitle(f'Top {plotncolors} colors ({ncolors} colors mode)')
        for i in range(plotncolors):
            ax.add_patch(patches.Rectangle((i, 0), 1, 1, facecolor=sorted_data[i][0].lower()))
        plt.xlim(0, plotncolors)
        plt.axis('off')  # H
        plt.show()
    
    return colorscheme

def get_colors(img, plot=False, plotncolors=5, clusterfactor = 50):
    """
    Extracts and optionally plots the most frequent colors in an image.

    This function reshapes the image to a list of RGB values, clusters these colors 
    based on the specified cluster factor to reduce the number of unique colors, and 
    returns the most common colors. Optionally, it can plot the top `plotncolors` colors 
    as rectangles.

    Parameters:
    - img (numpy.ndarray): The input image as a NumPy array of shape (height, width, 3).
    - plot (bool): If True, a plot of the top `plotncolors` colors is displayed. Default is False.
    - plotncolors (int): The number of top colors to plot. Default is 5.
    - clusterfactor (int): The factor by which to cluster similar colors. Default is 50.

    Returns:
    - numpy.ndarray: An array of RGB values representing the most frequent colors.
    """
    
    # Reshape the image array to a list of RGB values
    data = img[:, :, :3].reshape(img.shape[0] * img.shape[1], 3)
    
    # Handle alpha channel
    if img.shape[2] == 4:
        alpha = img[:, :, 3]
        # Exclude completely transparent pixels (alpha == 0) from distance calculation
        mask = alpha > 100 
        mask = mask.ravel()
        data = np.array(data)[mask]
        
    # Apply clustering by rounding the RGB values based on clusterfactor
    data = data // clusterfactor
    data = data.astype(int)
    data = data * clusterfactor
    
    # Convert to DataFrame to use value_counts for finding the most frequent colors
    df = pd.DataFrame(data, columns=['R', 'G', 'B'])
    sorted_data = df.value_counts().index.to_numpy()
    
    if plot:
        # Plot the top `plotncolors` colors
        fig, ax = plt.subplots()
        plt.suptitle(f'Top {plotncolors} colors')
        for i in range(min(plotncolors, len(sorted_data))):
            ax.add_patch(patches.Rectangle((i, 0), 1, 1, facecolor=[x/255 for x in sorted_data[i]]))
        plt.xlim(0, plotncolors)
        plt.axis('off')  # Hide axes
        plt.show()
    
    return sorted_data

###############################################################################
#                                                                             #
#                                   DEBUG                                     #
#                                                                             #
###############################################################################

if __name__ == '__main__':
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "panda.jpg"
    sample_img = "/home/giulio/Repositories/pyaesthetics/docs/examples/pyaesthetics_small.png"

    
    # Read and preprocess the sample image
    img = cv2.imread(sample_img, cv2.IMREAD_UNCHANGED)
    img = cv2.cvtColor(img, cv2.COLOR_BGRA2RGBA)
    
    # Display the image
    plt.imshow(img)
    
    
    # Calculate and print the color scheme using 16 W3C colors and plot the results
    results = get_colors_w3c(img, ncolors=16, plot=True, plotncolors=5)
    print("Color scheme of the image is:", results)

    # Calculate and print the color scheme using 16 W3C colors and plot the results
    results = get_colors_w3c(img, ncolors=140, plot=True, plotncolors=5)
    print("Color scheme of the image is:", results)

    # Calculate and print the color scheme using RGB colors (reduced to cluster similar colors) and plot the results
    results = get_colors(img, plot=True, plotncolors=5, clusterfactor = 10)
    print("Color scheme of the image is:", results)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module contains functions to evaluate the brightness of an image.
It includes a converter for sRGB to RGB and evaluation of relative luminance according to
BT.709 and BT.601 standards.

Created on Mon Apr 16 22:40:46 2018
Last edited on Fri Aug 2 11:34:10 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

###############################################################################
#                                                                             #
#                                   Libraries                                 #
#                                                                             #
###############################################################################

import os  # To handle filesystem files
import cv2  # For image manipulation
import numpy as np  # Numerical computation
import pandas as pd

###############################################################################
#                                                                             #
#                              Brightness                                     #
#                                                                             #
###############################################################################

""" This section handles brightness estimation. """
    
def relativeluminance_bt709(img):
    """ 
    This function evaluates the brightness of an image by means of Y, where Y is evaluated as:
            
    Y = 0.7152G + 0.0722B + 0.2126R
    B = mean(Y)
        
    :param img: image to analyze, in RGB
    :type img: numpy.ndarray
    :return: mean brightness
    :rtype: float
    """
    # Flatten the image array and reshape it to separate color channels
    img = np.array(img).flatten()
    img = img.reshape(int(len(img) / 3), 3)
    img = np.transpose(img)
    
    # Calculate the mean brightness using BT.709 standard
    B = np.mean(img[0]) * 0.2126 + np.mean(img[1]) * 0.7152 + np.mean(img[2]) * 0.0722
    
    return B  # Return the brightness index

def relativeluminance_bt601(img):
    """ 
    This function evaluates the brightness of an image by means of Y, where Y is evaluated as:
            
    Y = 0.587G + 0.114B + 0.299R
    B = mean(Y)
        
    :param img: image to analyze, in RGB
    :type img: numpy.ndarray
    :return: mean brightness
    :rtype: float
    """
    # Flatten the image array and reshape it to separate color channels
    img = np.array(img).flatten()
    img = img.reshape(int(len(img) / 3), 3)
    img = np.transpose(img)
    
    # Calculate the mean brightness using BT.601 standard
    B = np.mean(img[0]) * 0.299 + np.mean(img[1]) * 0.587 + np.mean(img[2]) * 0.114
    
    return B  # Return the brightness index

###############################################################################
#                                                                             #
#                                  DEBUG                                      #
#                                                                             #
###############################################################################

if __name__ == '__main__':
    import utils  # Importing utility functions
    
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "panda.jpg"
    
    # Read and preprocess the sample image
    img = cv2.imread(sample_img)
    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    img = utils.sRGB2RGB(img)
    
    # Calculate and print brightness using BT.709 and BT.601 standards
    print(relativeluminance_bt709(img))  
    print(relativeluminance_bt601(img))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module is used to evaluate the self-similarity and anisotropy of an image.
It is based on Braun, J., Amirshahi, S. A., Denzler, J., & Redies, C. (2013).
Statistical image properties of print advertisements, visual artworks, and images of architecture.
Frontiers in Psychology, 4, 808.

Created on Sat Aug 3 11:34:57 2024
Last edited on Sat Aug 3 14:09:43 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

import os
import cv2
import numpy as np
import matplotlib.pyplot as plt
from skimage.feature import hog
from skimage import exposure
from tqdm import tqdm

try:
    from . import utils
except:
    import utils

###############################################################################
#                                                                             #
#                             Self Similarity                                 #
#                                                                             #
###############################################################################

def preprocess_image(image_path):
    """
    Open an image, convert it to LAB color space, split into L, a, and b channels, 
    and calculate the gradient of each channel. Then merge the gradients by taking
    the maximum value for each pixel from the three gradients.
    
    :param image_path: Path to the input image.
    :return: Merged gradient image.
    """
    # Read the input image
    image = cv2.imread(image_path)
    if image is None:
        raise FileNotFoundError(f"Image not found at path: {image_path}")
    
    # Resize the image to a standard size
    resized_image = cv2.resize(image, (1024, 1024), interpolation=cv2.INTER_CUBIC)
    
    # Convert the image to LAB color space
    lab_image = cv2.cvtColor(resized_image, cv2.COLOR_BGR2Lab)
    
    # Split the LAB image into L, a, and b channels
    L, a, b = cv2.split(lab_image)
    
    # Calculate the gradient for each channel
    grad_L = utils.calculate_gradient(L)
    grad_a = utils.calculate_gradient(a)
    grad_b = utils.calculate_gradient(b)
    
    # Merge the gradients by taking the maximum value for each pixel
    merged_gradient = np.maximum(np.maximum(grad_L, grad_a), grad_b)
    
    return merged_gradient

def calculate_hog(image, orientations=16, pixels_per_cell=(16, 16), cells_per_block=(2, 2), visualize=True):
    """
    Calculate the Histogram of Oriented Gradients (HOG) for a given image.
    
    :param image: Input image.
    :param orientations: Number of orientation bins.
    :param pixels_per_cell: Size (in pixels) of a cell.
    :param cells_per_block: Number of cells in each block.
    :param visualize: Whether to return an image of the HOG.
    :return: HOG feature vector and HOG image (if visualize is True).
    """
    hog_features, hog_image = hog(image, orientations=orientations, pixels_per_cell=pixels_per_cell,
                                  cells_per_block=cells_per_block, visualize=visualize, block_norm='L2-Hys')
    return hog_features, hog_image

def bin_hog_features(hog_features, n_bins):
    """
    Bin the HOG features into n bins.
    
    :param hog_features: HOG feature vector.
    :param n_bins: Number of bins.
    :return: Binned HOG feature vector.
    """
    # Calculate the size of each bin
    bin_size = len(hog_features) // n_bins
    
    # Normalize the HOG features
    hog_features = hog_features / sum(hog_features)
    
    # Sum the HOG features into n bins
    binned_hog_features = np.sum(hog_features.reshape(bin_size, n_bins), axis=0)
    
    return binned_hog_features

def plot_hog_histogram(binned_hog_features, n_bins):
    """
    Plot the histogram of the binned HOG features.
    
    :param binned_hog_features: Binned HOG feature vector.
    :param n_bins: Number of bins.
    """
    plt.figure(figsize=(10, 6))
    plt.bar(range(n_bins), binned_hog_features, align='center', alpha=0.7)
    plt.xlabel('Bin')
    plt.ylabel('Sum of Magnitudes')
    plt.title('Histogram of Oriented Gradients (HOG) with 16 Bins')
    plt.show()

def get_self_similarity(image, methods=['ground', 'parent', 'neighbors', 'anisotropy']):
    """
    Calculate the self-similarity and anisotropy of an image using HOG features.

    :param image: Input image.
    :param methods: List of methods to calculate self-similarity. Options are 'ground', 'parent', 'neighbors', 'anisotropy'.
    :return: Dictionary of self-similarity measures.
    """
    results = {}
    
    # Resize the image to a standard size
    image = cv2.resize(image, (1024, 1024), interpolation=cv2.INTER_CUBIC)
    
    # Convert the image to LAB color space
    lab_image = cv2.cvtColor(image, cv2.COLOR_BGR2Lab)
    
    # Split the LAB image into L, a, and b channels
    L, a, b = cv2.split(lab_image)
    
    # Calculate the gradient for each channel
    grad_L = utils.calculate_gradient(L)
    grad_a = utils.calculate_gradient(a)
    grad_b = utils.calculate_gradient(b)
    
    # Merge the gradients by taking the maximum value for each pixel
    merged_gradient = np.maximum(np.maximum(grad_L, grad_a), grad_b)
    
    # Calculate HOG features for the merged gradient
    hog_features, hog_image = hog(merged_gradient, orientations=16, pixels_per_cell=(16, 16),
                                  cells_per_block=(2, 2), visualize=True, block_norm='L2-Hys')
    
    hists = []
    
    print('Computing HOGs. This is going to take some time.')
    nLevels = 4  # Number of levels in the pyramid
    for level in tqdm(range(nLevels)):
        this_level = []
        
        h, w = merged_gradient.shape
        hn = h // (2**level)
        wn = w // (2**level)
        
        for i in range(0, h, hn):
            for j in range(0, w, wn):
                partial_image = merged_gradient[i:i+hn, j:j+wn]
                hog_features, hog_image = calculate_hog(partial_image, orientations=16)
                
                # Bin the HOG features into n bins
                hist = bin_hog_features(hog_features, 16)
                this_level.append(hist)
                
        hists.append(np.array(this_level))
    
    # Ground method
    if 'ground' in methods:
        ground = hists[0].flatten()
        results['ground'] = np.median([(np.sum([min(ground[n], y[n]) for n in range(16)])) for y in hists[3]]).astype(float)

    # Parent approach
    if 'parent' in methods:
        temp = []
        for i in range(len(hists[-2])):
            nodes = utils.find_child_nodes(i)
            temp.append(np.median([(np.sum([min(hists[2][i][n], y[n]) for n in range(16)])) for y in hists[3][nodes]]))
        results['parent'] = np.median(temp).astype(float)
    
    # Neighbors approach
    if 'neighbors' in methods:
        temp = []
        for i in range(len(hists[3])):
            nodes = utils.find_neighbors(i)
            temp.append(np.median([(np.sum([min(hists[3][i][n], y[n]) for n in range(16)])) for y in hists[3][nodes]]))
        results['neighbors'] = np.median(temp).astype(float)
    
    # Anisotropy
    if 'anisotropy' in methods:
        results['anisotropy'] = np.std(hists[-1]).astype(float)
    
    return results

###############################################################################
#                                                                             #
#                                  DEBUG                                      #
#                                                                             #
###############################################################################

if __name__ == "__main__":
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "panda.jpg"
    
    # Read the sample image
    image = cv2.imread(sample_img)
    
    # Calculate self-similarity
    results = get_self_similarity(image)
    
    # Print the results
    print(results)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This file contains a class and functions to perform a Quadratic Tree decomposition
of an image and to visually inspect it.

Created on Mon Apr 16 11:49:45 2018
Last edited on Fri Aug 2 11:46:24 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

import os # to handle filesystem files
import cv2 # for image manipulation
import numpy as np # numerical computation
import matplotlib.pyplot as plt # for data visualization
import matplotlib.patches as patches # for drawing rectangles in the plot
import math 

###############################################################################
#                                                                             #
#                      Quadratic Tree Decomposition                           #
#                                                                             #
###############################################################################
""" This section handles Quadratic Tree Decomposition. """

class quadTree:
    """ 
    This class performs a QuadTree decomposition of an image. 
    
    During initialization, the QuadTree decomposition is done and results are stored in `self.blocks` 
    as a list containing [x, y, height, width, Std]. To visualize the results, use the `plot()` method.
    The total number of blocks is stored in `self.nblocks`, while the standardized complexity (number of
    blocks divided by the total possible number of blocks is stored in `self.standardized_complexity`. 
    Standardized complexity may range from 0 to 1 (with one being the highest complexity possible).
    The minSize parameter can be automatically adjusted to the common divisor of image width and height closest 
    to the imputted minSize via the `autoadjust` parameter (default to False). The adjusted `minSize` can be 
    obtained (if `autoadjust` = True) by accessing self.minSize.
    """
    
    def plot(self, edgecolor="red", facecolor="none", linewidth=1):
        """ 
        Generate a graphical representation of the QuadTree decomposition. 
        
        :param edgecolor: Color of the rectangle edges, default is red
        :type edgecolor: string
        :param facecolor: Color used for rectangle fills. Default is none.
        :type facecolor: string
        :param linewidth: Width of the rectangles' borders in pixels. Default is 1.
        :type linewidth: int
        :return: Plot with image and leaves of the QuadTree Decomposition
        """
        plt.figure() # Create a new figure
        fig = plt.imshow(self.img) # Plot the original image
        for block in self.blocks: # For each block of results
            rect = patches.Rectangle(
                (block[0], block[1]), # Position (x, y) of the rectangle
                block[3], # Width of the rectangle
                block[2], # Height of the rectangle
                linewidth=linewidth, # Border width
                edgecolor=edgecolor, # Border color
                facecolor=facecolor # Fill color
            )
            fig.axes.add_patch(rect) # Add rectangle to the plot
            plt.title("QuadTree Decomposition") # Set the plot title
            plt.xticks([],[])
            plt.yticks([],[])
        plt.show() # Display the plot
        
    def quad_tree_decomposition(self, img, x, y, minStd, minSize):
        """ 
        Evaluate the mean and standard deviation of an image block, and decide whether to perform 
        further splits of the block.
        
        :param img: Image to analyze
        :type img: numpy.ndarray
        :param x: X offset of the block to analyze
        :type x: int
        :param y: Y offset of the block to analyze
        :type y: int
        :param minStd: Standard deviation threshold for subsequent splitting
        :type minStd: int
        :param minSize: Size threshold for subsequent splitting, in pixels
        :type minSize: int
        """
        h, w = img.shape # Get the height and width of the image
        mean, std = cv2.meanStdDev(img) # Compute the mean and standard deviation
        mean, std = [int(mean.item()), int(std.item())]  
        if std >= minStd and max(h/2, w/2) >= minSize: # Check if std and size thresholds are met
            # Decide whether to split along X or Y axis
            if w >= h: # Split along the X axis
                w2 = int(w / 2) # Compute new width
                img1 = img[0:h, 0:w2] # Create the left subimage
                img2 = img[0:h, w2:] # Create the right subimage
                # Recursively perform QuadTree decomposition
                self.quad_tree_decomposition(img1, x, y, minStd, minSize)
                self.quad_tree_decomposition(img2, x + w2, y, minStd, minSize)
            else: # Split along the Y axis
                h2 = int(h / 2) # Compute new height
                img1 = img[0:h2, 0:] # Create the upper subimage
                img2 = img[h2:, 0:] # Create the lower subimage
                # Recursively perform QuadTree decomposition
                self.quad_tree_decomposition(img1, x, y, minStd, minSize)
                self.quad_tree_decomposition(img2, x, y + h2, minStd, minSize)
        else:
            # Add the block to the results if it does not meet the splitting criteria
            self.blocks.append([x, y, h, w, std])
            self.nblocks = len(self.blocks)
            self.standardized_complexity = self.nblocks / self.max_n_blocks
            
    def __init__(self, img, minStd, minSize, autoadjust = False):
        """ 
        Initialize the QuadTree decomposition analysis. 
        
        :param img: Image to analyze
        :type img: numpy.ndarray
        :param minStd: Standard deviation threshold for subsequent splitting
        :type minStd: int
        :param minSize: Size threshold for subsequent splitting, in pixels
        :type minSize: int
        """
        self.blocks = [] # Initialize the results list
        self.img = img # Assign the image
        self.params = [minStd, minSize] # Set the parameters
        self.minSize = minSize
        if(autoadjust):
            self.minSize = closest_common_divisor(self.img.shape[0], self.img.shape[1], minSize)
            print('Minimum size has been adjusted automatically. The new minSize is ', self.minSize)
        self.max_n_blocks = self.max_quadtree_leaves(self.img.shape[0], self.img.shape[1], self.minSize)
        self.nblocks = 0
        self.standardized_complexity = 0 
        
        self.quad_tree_decomposition(self.img, 0, 0, minStd, self.minSize) # Start the decomposition

    def max_quadtree_leaves(self, width, height, min_leaf_size):
        """
        Calculate the maximum number of leaves in a quadtree decomposition of an image.
    
        :param width: Width of the image in pixels.
        :type width: int
        :param height: Height of the image in pixels.
        :type height: int
        :param min_leaf_size: Minimum size of each side of a leaf in pixels.
        :type min_leaf_size: int
        :return: Maximum number of leaves in the quadtree decomposition.
        :rtype: int
        """
        
        # Calculate maximum depth for width and height
        depth_width = 1
        while width /2/ 2**depth_width >= min_leaf_size:
            depth_width+=1
            
        
        # Calculate maximum depth for width and height
        depth_height = 1
        while height/2 / 2**depth_height >= min_leaf_size:
            depth_height+=1
        
        return((2**depth_height) * (2**depth_width))

def common_divisors(n, m):
    """
    Find all common divisors of n and m.

    :param n: First number.
    :type n: int
    :param m: Second number.
    :type m: int
    :return: A set of common divisors of n and m.
    :rtype: set
    """
    def get_divisors(x):
        divisors = set()
        for i in range(1, int(x**0.5) + 1):
            if x % i == 0:
                divisors.add(i)
                divisors.add(x // i)
        return divisors

    divisors_n = get_divisors(n)
    divisors_m = get_divisors(m)
    return divisors_n.intersection(divisors_m)

def closest_common_divisor(n, m, k):
    """
    Find the common divisor of n and m that is closest to k.

    :param n: First number.
    :type n: int
    :param m: Second number.
    :type m: int
    :param k: Target value.
    :type k: int
    :return: The common divisor closest to k.
    :rtype: int
    """
    common_divs = common_divisors(n, m)
    closest_divisor = min(common_divs, key=lambda x: abs(x - k))
    return closest_divisor

###############################################################################
#                                                                             #
#                                  DEBUG                                      #
#                                                                             #
###############################################################################
        
""" For debug purposes, this section reads a sample image and performs QuadTree decomposition. """

if __name__ == '__main__':
    # Path to a sample image
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "panda.jpg"

    minStd = 20 #Minimum standard deviation for each block
    minSize = 30 # Minimum size of each block in pixels
    
    imgcolor = cv2.imread(sample_img) # Read the image in color for plotting purposes
    img = cv2.imread(sample_img, 0) # Read the image in grayscale
    
    # Create an instance of the quadTree class and perform decomposition
    mydecomposition = quadTree(img, minStd, minSize, autoadjust=True) 
    mydecomposition.plot() # Visual inspection of the results
    print(len(mydecomposition.blocks), mydecomposition.nblocks, mydecomposition.standardized_complexity)



from . import analysis
from . import brightness
from . import colordetection
from . import colorfulness
from . import contrast
from . import facedetection
from . import linesdetection
from . import quadtreedecomposition
from . import saturation
from . import selfsimilarity
from . import spacebaseddecomposition
from . import symmetry
from . import utils
from . import visualcomplexity

# This is used to print out the package version
__version__ = '0.0.8.4' #Version Control

print("Thank you for using pyaesthetics. If you use it in your work, please cite:")
print("Gabrieli, G., Bornstein, M. H., Setoh, P., & Esposito, G. (2023). Machine learning estimation of users’ implicit and explicit aesthetic judgments of web-pages. Behaviour & Information Technology, 42(4), 392-402.")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module contains functions to evaluate the contrast of an image using either 
RMS contrast or Michelson contrast.

Created on Wed Apr 10 11:49:45 2018
Last edited on Fri Aug  2 11:46:24 2024
  
@author: Giulio Gabrieli (gack94@gmail.com)
"""

###############################################################################
#                                                                             #
#                                   Libraries                                 #
#                                                                             #
###############################################################################

import os # to handle filesystem files
import cv2 # for image manipulation
import numpy as np # numerical computation
import pandas as pd # for data manipulation (not used in this script, but imported)

###############################################################################
#                                                                             #
#                              Contrast Estimation                           #
#                                                                             #
###############################################################################

""" This section handles contrast estimation. """

def contrast_rms(img):
    """ 
    This function evaluates the RMS (Root Mean Square) contrast of an image.
    The RMS contrast is calculated as the standard deviation of pixel intensities 
    in the grayscale version of the image.
    
    :param img: image to analyze, in RGB
    :type img: numpy.ndarray
    :return: RMS contrast 
    :rtype: float
    """
    
    # Convert the image to grayscale and normalize pixel values to the range [0, 1]
    img_grey = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY) / 255
    
    # Compute the standard deviation of pixel values in the grayscale image
    contrast = img_grey.std()
    
    # Alternative method to compute RMS contrast (commented out for reference)
    # img_s = img_grey - img_grey.mean()
    # img_s = img_s**2
    # contrast2 = np.sqrt(img_s.sum() / (img_s.shape[0] * img_s.shape[1]))
    # print(contrast, contrast2)
    
    return contrast

def contrast_michelson(img):
    """ 
    This function evaluates the Michelson contrast of an image.
    The Michelson contrast is calculated based on the luminance channel (Y) of the YUV color space:
    
    Michelson contrast = (maxY - minY) / (maxY + minY)
    
    :param img: image to analyze, in RGB
    :type img: numpy.ndarray
    :return: Michelson contrast
    :rtype: float
    """
    
    # Convert the image to YUV color space and extract the luminance channel (Y)
    Y = cv2.cvtColor(img, cv2.COLOR_BGR2YUV)[:,:,0]

    # Compute the minimum and maximum values of the Y channel
    minY = float(np.min(Y))
    maxY = float(np.max(Y))
    
    # Compute the Michelson contrast
    contrast = (maxY - minY) / (maxY + minY)

    return contrast

###############################################################################
#                                                                             #
#                                  DEBUG                                      #
#                                                                             #
###############################################################################
        
""" For debug purposes, this section reads sample images and calculates their contrast. """

if __name__ == '__main__':
    
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "panda.jpg"
    
    # Read the image from file
    img = cv2.imread(sample_img)
    
    # Print the RMS and Michelson contrast values for the image
    print("RMS Contrast:", contrast_rms(img))  
    print("Michelson Contrast:", contrast_michelson(img))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module is used to evaluate the visual complexity of an image. 

Created on Sat Aug  3 11:34:57 2024
Last edited on Sat Aug 13:46:43 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

import os
import cv2  # OpenCV library for image processing
import matplotlib.pyplot as plt  # Matplotlib for plotting images
from PIL import Image  # Python Imaging Library for image processing
import numpy as np  # NumPy for numerical operations

# Attempt to import internal modules of pyaesthetics, handling both relative and absolute imports
try:
    from . import quadtreedecomposition
    from . import utils
except:
    import quadtreedecomposition
    import utils


###############################################################################
#                                                                             #
#                            Visual Complexity                                #
#                                                                             #
###############################################################################

def get_visual_complexity_quadtree(image, minStd, minSize, standardized=True, autoadjust=False):
    """
    Calculate the visual complexity of an image using quadtree decomposition.
    It can return the standardized (default) visual complexity, with 1 being the 
    highest complexity possible, or unstandardized (which is the number of leaves
    resulting from the quadratic tree decomposition).

    :param image: Input image (grayscale).
    :type image: numpy.ndarray
    :param minStd: Minimum standard deviation for splitting blocks.
    :type minStd: int
    :param minSize: Minimum size of blocks.
    :type minSize: int
    :param standardized: Whether to return standardized complexity.
    :type standardized: bool
    :param autoadjust: Whether to automatically adjust the minSize parameter.
    :type autoadjust: bool
    :return: Standardized complexity if `standardized` is True, otherwise the number of blocks.
    :rtype: float or int
    """
    # Perform quadtree decomposition
    quadtree = quadtreedecomposition.quadTree(image, minStd, minSize, autoadjust)
    
    if(standardized):
        return(quadtree.standardized_complexity)
    else:
        # Return the number of blocks
        return(quadtree.nblocks)

def get_visual_complexity_gradient(image):
    """
    Calculate the visual complexity of an image using gradient magnitude.

    :param image: Input image (color).
    :return: Mean of the merged gradient magnitudes.
    """
    # Resize the image to 1024x1024
    image = cv2.resize(image, (1024, 1024), interpolation=cv2.INTER_CUBIC)
    
    # Convert the image to LAB color space
    lab_image = cv2.cvtColor(image, cv2.COLOR_BGR2Lab)
    
    # Split the LAB image into L, a, and b channels
    L, a, b = cv2.split(lab_image)
    
    # Calculate the gradient for each channel
    grad_L = utils.calculate_gradient(L)
    grad_a = utils.calculate_gradient(a)
    grad_b = utils.calculate_gradient(b)
    
    # Merge the gradients by taking the maximum value for each pixel
    merged_gradient = np.maximum(np.maximum(grad_L, grad_a), grad_b)
    
    # Return the mean of the merged gradient magnitudes
    return(np.mean(merged_gradient).astype(float))
    
def get_visual_complexity_weight(path_to_image):
    """
    Calculate the visual complexity of an image based on its file size.

    :param path_to_image: Path to the image file.
    :return: File size in bytes.
    """
    # Return the file size in bytes
    return(os.stat(path_to_image).st_size)

###############################################################################
#                                                                             #
#                                  DEBUG                                      #
#                                                                             #
###############################################################################

if __name__ == '__main__':
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Set the data path to use sample images
    datafolder = os.path.join(basepath, "../share/data/")
    
    # Path to a sample image
    sample_img_path = os.path.join(datafolder, "panda.jpg")

    # Read the sample image in color and grayscale
    image = cv2.imread(sample_img_path)
    image_bw = cv2.imread(sample_img_path, 0)

    # Analyze the sample image using quadtree decomposition and gradient magnitude methods
    qt_complexity = get_visual_complexity_quadtree(image_bw, minStd=10, minSize=20)
    grad_complexity = get_visual_complexity_gradient(image)
    
    # Print the analysis results
    print(qt_complexity, grad_complexity)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module contains function to evaluate the colorfulness of an image in both the HSV and RGB color spaces.

Created on Mon Apr 16 11:49:45 2018
Last edited on Fri Aug 2 11:44:12 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

import os #to handle filesystem files
import cv2 #for image manipulation
import numpy as np #numerical computation

###############################################################################
#                                                                             #
#                              Colorfulness                                   #
#                                                                             #
###############################################################################

""" This section handles colorfulness estimation. """

def colorfulness_hsv(img):
    """ 
    This function evaluates the colorfulness of a picture using the formula described in Yendrikhovskij et al., 1998.
    Input image is first converted to the HSV color space, then the S values are selected.
    Ci is evaluated with a sum of the mean S and its std, as in:
        
    Ci = mean(Si) + std(Si)

    :param img: image to analyze, in RGB
    :type img: numpy.ndarray
    :return: colorfulness index
    :rtype: float
    """
    
    # Convert the image to the HSV color space
    img = cv2.cvtColor(img, cv2.COLOR_RGB2HSV)
    
    # Initialize a list to store the Saturation values
    S = []
    
    # Extract the Saturation value for each pixel
    for row in img:
        for pixel in row:
            S.append(pixel[1])
    
    # Evaluate the colorfulness index
    C = np.mean(S) + np.std(S)
    return C

def colorfulness_rgb(img):
    """ 
    This function evaluates the colorfulness of a picture using Metric 3 described in Hasler & Suesstrunk, 2003.
    Ci is evaluated with as:
        
    Ci = std(rgyb) + 0.3 mean(rgyb)   [Equation Y] 
    std(rgyb) = sqrt(std(rg)^2 + std(yb)^2)
    mean(rgyb) = sqrt(mean(rg)^2 + mean(yb)^2)
    rg = R - G
    yb = 0.5(R + G) - B

    :param img: image to analyze, in RGB
    :type img: numpy.ndarray
    :return: colorfulness index
    :rtype: float
    """
    
    # Initialize lists to store the RGB values
    R = []
    G = []
    B = []
    
    # Extract the RGB values for each pixel
    for row in img:
        for pixel in row:
            R.append(int(pixel[0]))
            G.append(int(pixel[1]))
            B.append(int(pixel[2]))
    
    # Evaluate rg and yb
    rg = [R[x] - G[x] for x in range(len(R))]
    yb = [0.5 * (R[x] + G[x]) - B[x] for x in range(len(R))]
    
    # Compute the standard deviation and mean of rgyb
    stdRGYB = np.sqrt((np.std(rg) ** 2) + (np.std(yb) ** 2))
    meanRGYB = np.sqrt((np.mean(rg) ** 2) + (np.mean(yb) ** 2))
    
    # Compute the colorfulness index
    C = stdRGYB + 0.3 * meanRGYB
    return C

###############################################################################
#                                                                             #
#                                  DEBUG                                      #
#                                                                             #
###############################################################################
        
""" For debug purposes."""

if __name__ == '__main__':
    
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "panda.jpg"
    
    # Read and convert the image
    img = cv2.imread(sample_img)
    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    
    # Print the colorfulness indices
    print(colorfulness_hsv(img))    
    print(colorfulness_rgb(img))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module contains functions to compute the degree of symmetry of an image.
- Symmetry by QuadTree Decomposition

Created on Mon Apr 16 11:49:45 2018
Last edited on Fri Aug 2 12:05:23 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

import os # to handle filesystem files
import cv2 # for image manipulation
import numpy as np # numerical computation
import matplotlib.pyplot as plt # for data visualization
import matplotlib.patches as patches # for legends and rectangle drawing in QTD

# Attempt to import internal modules of pyaesthetics, handling both relative and absolute imports
try:
    from . import quadtreedecomposition
    from . import utils
except:
    import quadtreedecomposition
    import utils

###############################################################################
#                                                                             #
#                                  Symmetry                                   #
#                                                                             #
###############################################################################
        
def get_symmetry(img, minStd, minSize, plot=False):
    """ Returns the degree of symmetry (0-100) between the left and right side of an image.
    
    :param img: Image to analyze.
    :type img: numpy.ndarray
    :param minStd: Std deviation threshold for splitting.
    :type minStd: int
    :param minSize: Size threshold for splitting, in pixels.
    :type minSize: int
    :param plot: Whether to plot the QuadTree decomposition of each half.
    :type plot: bool
    :return: Degree of vertical symmetry between 0 and 1.
    :rtype: float
    """
    h, w = img.shape
    if h % 2 != 0:
        img = img[:-1, :] # Crop image to even height
    if w % 2 != 0:
        img = img[:, :-1] # Crop image to even width
        
    left = img[:, :int(w / 2)] # Left half of the image
    right = np.flip(img[:, int(w / 2):], 1) # Right half, flipped horizontally
    left = quadtreedecomposition.quadTree(left, minStd, minSize) # Decompose left half
    right = quadtreedecomposition.quadTree(right, minStd, minSize) # Decompose right half
    
    if plot:
        left.plot() # Plot decomposition of left half
        right.plot() # Plot decomposition of right half
        
    counter = 0
    total = len(right.blocks) + len(left.blocks)
    
    # Count matching blocks between left and right halves
    for block in right.blocks:
        for block2 in left.blocks:
            if block[0:4] == block2[0:4]:
                counter += 1
                
    return (counter / total * 2) # Return degree of symmetry

###############################################################################
#                                                                             #
#                                  DEBUG                                      #
#                                                                             #
###############################################################################
        
""" For debug purposes. """

if __name__ == '__main__':
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "panda.jpg"
    
    minStd = 5 # Min STD of each block
    minSize = 20 # Min size of each block
    
    imgcolor = cv2.imread(sample_img) # Read the image in color for plotting purposes
    img = cv2.imread(sample_img, 0) # Read the image in grayscale
    # Symmetry using QuadTree decomposition
    h, w = img.shape
    print(get_symmetry(img, minSize, minStd, plot=True)) # Print the degree of symmetry



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This modules contains different utilities  that are used across different modules in pyaesthetics.

Created on Fri Aug 2 10:51:17 2024
Last Edited on Sat Aug 3 11:53:30 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

import os

from PIL import Image #to handle filesystem files
import cv2 #for image manipulation
import numpy as np #numerical computation
import pandas as pd 
import pytesseract  # Pytesseract for Optical Character Recognition (OCR)
from imutils.perspective import four_point_transform
import matplotlib.pyplot as plt 

try:
    from . import analysis
except:
    import analysis

def sRGB2RGB(img):
    """ this function converts a sRGB img to linear RGB values.
    
        It loops through each pixel, and apply a conversion to pass from sRGB to linear RGB value.
        
    
        :param img: image to analyze, in sRGB
        :type img: numpy.ndarray
        :return: image to analyze, in RGB
        :rtyipe: numpy.ndarray
    """

    img = img.flatten()
    def converter(p):
        if(p < 0.04045):
            return(p/3294.6)
        else:
            return((((p/255) + 0.055) / 1.055)**2.4)

    newimg = pd.Series(img).apply(converter).to_numpy()
    return(newimg)

def find_parent_node(i):
    """
    Find the parent node in a 4x4 matrix given an index in an 8x8 matrix.

    :param i: Index in the 8x8 matrix.
    :return: Parent node index in the 4x4 matrix.
    """
    if i < 0 or i >= 64:
        raise ValueError("Index must be in the range [0, 63]")

    # Calculate the row and column in the 8x8 matrix
    row = i // 8
    col = i % 8

    # Determine the 2x2 block's row and column in the 4x4 matrix
    block_row = row // 2
    block_col = col // 2

    # Calculate the index in the 4x4 matrix
    parent_index = block_row * 4 + block_col

    return parent_index

def find_child_nodes(parent_index):
    """
    Find the child nodes in an 8x8 matrix given a parent node index in a 4x4 matrix.

    :param parent_index: Index in the 4x4 matrix.
    :return: List of child node indices in the 8x8 matrix.
    """
    if parent_index < 0 or parent_index >= 16:
        raise ValueError("Parent index must be in the range [0, 15]")

    # Calculate the row and column in the 4x4 matrix
    parent_row = parent_index // 4
    parent_col = parent_index % 4

    # Determine the top-left corner of the 2x2 block in the 8x8 matrix
    top_left_row = parent_row * 2
    top_left_col = parent_col * 2

    # List the indices of the 4 child nodes in the 2x2 block
    child_indices = [
        top_left_row * 8 + top_left_col,       # Top-left
        top_left_row * 8 + top_left_col + 1,   # Top-right
        (top_left_row + 1) * 8 + top_left_col, # Bottom-left
        (top_left_row + 1) * 8 + top_left_col + 1 # Bottom-right
    ]

    return child_indices


def find_neighbors(index, size=8):
    """
    Find all neighboring cells for a given cell index in an NxN matrix.

    :param index: Index of the cell in the NxN matrix.
    :param size: Size of the matrix (default is 8 for an 8x8 matrix).
    :return: List of neighboring cell indices.
    """
    row = index // size
    col = index % size

    neighbors = []

    # Directions for neighbors: (row_offset, col_offset)
    directions = [
        (-1, 0),  # Top
        (1, 0),   # Bottom
        (0, -1),  # Left
        (0, 1),   # Right
        (-1, -1), # Top-left
        (-1, 1),  # Top-right
        (1, -1),  # Bottom-left
        (1, 1)    # Bottom-right
    ]

    for dr, dc in directions:
        r, c = row + dr, col + dc
        if 0 <= r < size and 0 <= c < size:
            neighbors.append(r * size + c)

    return neighbors

def calculate_gradient(image):
    """
    Calculate the gradient of an image using the Sobel operator.
    
    :param image: Input image (single channel).
    :return: Gradient magnitude of the image.
    """
    grad_x = cv2.Sobel(image, cv2.CV_64F, 1, 0, ksize=3)
    grad_y = cv2.Sobel(image, cv2.CV_64F, 0, 1, ksize=3)
    gradient_magnitude = cv2.magnitude(grad_x, grad_y)
    
    return gradient_magnitude


def runtest():
    """ This function runs a complete test to verify the integroty
            of the installation.

        :return: a funny string
        :rtype: string

    """
    basepath = os.path.dirname(os.path.realpath(__file__))
    # Path to a sample image for debugging   # Set the data path to use sample images
    datafolder = basepath + "/../share/data/"
    # Path to a sample image
    sampleImg = datafolder + "test.png"
    # Analyze the sample image using the 'complete' method
    print('Running test')
    analysis.analyze_image(sampleImg, method='complete')
    
    return('All good! Enjoy Pyaesthetics.')

def textdetection(img):
    """ This function uses pytesseract to get information about the presence of text in an image.

        :param img: image to analyze, in RGB
        :type img: numpy.ndarray
        :return: number of characters in the text
        :rtype: int

    """
    # Convert the image to grayscale
    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    # Save the image to a temporary file
    filename = "{}.png".format(os.getpid())
    cv2.imwrite(filename, img)
    # Perform OCR to extract text from the image
    text = pytesseract.image_to_string(Image.open(filename))
    # Remove the temporary file
    os.remove(filename)
    # Return the length of the extracted text
    return len(text)

###############################################################################
#                                                                             #
#                                  DEBUG                                      #
#                                                                             #
###############################################################################
""" For debug purposes."""

if __name__ == "__main__":
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/data/"
    
    # Path to a sample image
    sample_img = data_folder + "face1.png"
    
    img = cv2.imread(sample_img)
    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    
    runtest()
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module contains functions to evaluate the amount of straight and curved lines in an image.
This is a very experimental feature; usage is not recommended.

Created on Wed Jul 31 11:49:45 2024
Last edited on Fri Aug  2 11:46:24 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

###############################################################################
#                                                                             #
#                                   Libraries                                 #
#                                                                             #
###############################################################################

import os # to handle filesystem files
import cv2 # for image manipulation
import numpy as np # numerical computation
import pandas as pd # for data manipulation (not used in this script, but imported)
import math # for mathematical operations (not used in this script)
import matplotlib.pyplot as plt # for plotting images and results

###############################################################################
#                                                                             #
#                              Lines Detection                                #
#                                                                             #
###############################################################################

""" This section handles the detection of lines in images. """

def get_straight_lines(image, plot=False):
    """
    Detect straight lines in an image using the Hough Line Transform.

    :param image: The image to analyze, in RGB format.
    :type image: numpy.ndarray
    :param plot: Whether to plot the image with detected lines or not (default is False).
    :type plot: bool, optional
    :return: Detected straight lines.
    :rtype: list of numpy.ndarray or None
    """
    # Convert the image to grayscale
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    
    # Apply edge detection
    edges = cv2.Canny(gray, 25, 100, apertureSize=3)

    # Detect lines using the Hough Line Transform
    lines = cv2.HoughLinesP(edges, 1, np.pi/180, threshold=100, minLineLength=15, maxLineGap=10)
    
    if plot:
        # Draw the lines on the image
        if lines is not None:
            for line in lines:
                x1, y1, x2, y2 = line[0]
                cv2.line(image, (x1, y1), (x2, y2), (0, 255, 0), 2)

        # Display the result with detected lines
        plt.imshow(cv2.cvtColor(image, cv2.COLOR_BGR2RGB))
        plt.title('Detected Straight Lines')
        plt.show()

    return lines

def get_curved_lines(image, plot=False):
    """
    Detect curved lines (contours) in an image.

    :param image: The image to analyze, in RGB format.
    :type image: numpy.ndarray
    :param plot: Whether to plot the image with detected contours or not (default is False).
    :type plot: bool, optional
    :return: Detected contours.
    :rtype: list of numpy.ndarray
    """
    # Convert the image to grayscale
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

    # Apply edge detection
    edges = cv2.Canny(gray, 50, 150)

    # Find contours in the image
    contours, hierarchy = cv2.findContours(edges, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

    if plot:
        # Draw contours on the image
        cv2.drawContours(image, contours, -1, (255, 0, 0), 2)

        # Display the result with detected contours
        plt.imshow(cv2.cvtColor(image, cv2.COLOR_BGR2RGB))
        plt.title('Detected Contours')
        plt.show()
        
    return contours

def calculate_distance(x1, y1, x2, y2):
    """
    Calculate the distance between two points (x1, y1) and (x2, y2).

    :param x1: The x-coordinate of the first point.
    :type x1: float
    :param y1: The y-coordinate of the first point.
    :type y1: float
    :param x2: The x-coordinate of the second point.
    :type x2: float
    :param y2: The y-coordinate of the second point.
    :type y2: float
    :return: The distance between the two points.
    :rtype: float
    """
    return np.sqrt((x2 - x1) ** 2 + (y2 - y1) ** 2)

def get_lines_ratio(lines, curves):
    """
    Calculate the ratio of the total length of straight lines to the total length of curved lines in an image.

    :param lines: List of detected straight lines.
    :type lines: list of numpy.ndarray or None
    :param curves: List of detected contours.
    :type curves: list of numpy.ndarray
    :return: The ratio of total straight line length to total curved line length. Returns NaN if division by zero occurs.
    :rtype: float
    """
    # Initialize total length
    lines_total_length = 0
    curved_total_length = 0
    
    # Calculate the total length of straight lines
    if lines is not None:
        for line in lines:
            x1, y1, x2, y2 = line[0]
            length = calculate_distance(x1, y1, x2, y2)
            lines_total_length += length

    # Calculate the total length of curved lines (contours)
    for contour in curves:
        length = cv2.arcLength(contour, True)  # True indicates the contour is closed
        curved_total_length += length
         
    # Return the ratio of straight line length to curved line length, or NaN if division by zero occurs
    try:
        return lines_total_length / curved_total_length
    except ZeroDivisionError:
        return np.nan

def analyse_lines(image):
    """
    Analyze an image to determine the ratio of the total length of straight lines to the total length of curved lines.

    :param image: The image to analyze, in RGB format.
    :type image: numpy.ndarray
    :return: The ratio of total straight line length to total curved line length. Returns NaN if division by zero occurs.
    :rtype: float
    """
    lines = get_straight_lines(image)
    curves = get_curved_lines(image)
    ratio = get_lines_ratio(lines, curves)
    return ratio

###############################################################################
#                                                                             #
#                                  DEBUG                                      #
#                                                                             #
###############################################################################
        
""" For debug purposes, this section reads a sample image and calculates the ratio of straight to curved lines. """

if __name__ == '__main__':
    # Path to the sample image
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "panda.jpg"

    img = cv2.imread(sample_img)
    
    # Detect straight lines and curved lines (contours)
    lines = get_straight_lines(img)
    curves = get_curved_lines(img)
    
    # Calculate and print the ratio of straight to curved lines
    ratio = get_lines_ratio(lines, curves)
    print(ratio)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This is an entrypoint for the automatic analysis of images using pyaesthetics.


Created on Mon Apr 16 22:40:46 2018
Last edited on Fri Aug 2 09:53:57 2024

@author: Giulio Gabrieli (gack94@gmail.com)
"""

###############################################################################
#                                                                             #
#                                 Libraries                                   #
#                                                                             #
###############################################################################

import os
import cv2  # OpenCV library for image processing
import matplotlib.pyplot as plt  # Matplotlib for plotting images
import pytesseract  # Pytesseract for Optical Character Recognition (OCR)
from PIL import Image  # Python Imaging Library for image processing

# Attempt to import internal modules of pyaesthetics, handling both relative and absolute imports
try:
    from . import brightness
    from . import colordetection
    from . import colorfulness
    from . import contrast
    from . import facedetection
    from . import linesdetection
    from . import quadtreedecomposition
    from . import saturation
    from . import selfsimilarity
    from . import spacebaseddecomposition
    from . import symmetry
    from . import utils
    from . import visualcomplexity
except:
    import analysis
    import brightness
    import colordetection
    import colorfulness
    import contrast
    import facedetection
    import linesdetection
    import quadtreedecomposition
    import saturation
    import selfsimilarity
    import spacebaseddecomposition
    import symmetry
    import utils
    import visualcomplexity

###############################################################################
#                                                                             #
#                      Quadratic Tree Decomposition                           #
#                                                                             #
###############################################################################

def analyze_image(pathToImg, method='fast', resize=True, newSize=(600, 400), minStd=10, minSize=20):
    """ This function acts as an entry point for the automatic analysis of an image's aesthetic features.
        There are two methods available: 'fast' (default) and 'complete'.
    
        The 'fast' method performs the following analysis: 
        - brightness (BT709)
        - Visual Complexity (using quadtree decomposition)
        - Symmetry (using quadtree decomposition)
        - Colorfulness (in the RGB color scheme)
        - contrast (Root Mean Square (RMS) method)
        - saturation
        - ratio between straight and curved lines

        The 'complete' method performs the following analysis:
        - brightness (BT709 and BT601)
        - Visual Complexity (using quadtree decomposition)
        - Visual Complexity (using the weight method)
        - Symmetry (using quadtree decomposition)
        - Colorfulness (in HSV and RGB color schemes)
        - Face detection and number of faces
        - Color detection
        - Number of images within an image
        - Text to image ratio
        - Surface area occupied by text and images
        - contrast (Root Mean Square (RMS) and Michelson methods)
        - saturation
        - ratio between straight and curved lines

        :param pathToImg: path to the image to analyze
        :type pathToImg: str
        :param method: set the analysis method to use. Valid methods are 'fast' and 'complete'. Default is 'fast'.
        :type method: str
        :param resize: indicate whether to resize the image (reduces computational workload, this reducing processing time)
        :type resize: bool
        :param newSize: if the image has to be resized, this tuple indicates the new size of the image
        :type newSize: tuple
        :param minStd: minimum standard deviation for the quadtree decomposition
        :type minStd: int
        :param minSize: minimum size for the quadtree decomposition
        :type minSize: int
        :return: dictionary containing the analysis results
        :rtype: dict
    """

    resultdict = {}
    # Read the image from the provided path
    img = cv2.imread(pathToImg)
    
    # Convert the image to RGB format
    imageColor = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    # Read the image in grayscale
    imageBW = cv2.imread(pathToImg, 0)
    # Detect text in the image and store the result in the dictionary
    resultdict["Text"] = utils.textdetection(imageColor)
    # Keep a copy of the original size image
    imageColor_O = imageColor
    # Resize the image if requested
    if resize:
        imageBW = cv2.resize(imageBW, newSize, interpolation=cv2.INTER_CUBIC)
        imageColor = cv2.resize(imageColor, newSize, interpolation=cv2.INTER_CUBIC)
    
    # Convert sRGB to RGB using the brightness module
    imgsRGB2RGB = utils.sRGB2RGB(img)

    # Perform analysis based on the selected method
    if method == 'fast':
        # Fast method: fewer and quicker calculations
        resultdict["brightness_BT709"] = brightness.relativeluminance_bt709(imgsRGB2RGB)
        resultdict["VC_quadTree"] = visualcomplexity.get_visual_complexity_quadtree(imageBW, minStd, minSize)
        resultdict["Symmetry_QTD"] = symmetry.get_symmetry(imageBW, minStd, minSize)
        resultdict["Colorfulness_RGB"] = colorfulness.colorfulness_rgb(imageColor)
        resultdict["contrast_RMS"] = contrast.contrast_rms(imageColor)
        resultdict["saturation"] = saturation.saturation(imageColor)

    elif method == 'complete':
        # Complete method: more detailed and extensive calculations
        resultdict["brightness_BT709"] = brightness.relativeluminance_bt709(imgsRGB2RGB)
        resultdict["brightness_BT601"] = brightness.relativeluminance_bt601(imgsRGB2RGB)
        resultdict["VC_quadTree"] = visualcomplexity.get_visual_complexity_quadtree(imageBW, minStd, minSize)
        resultdict["VC_gradient"] = visualcomplexity.get_visual_complexity_gradient(imageColor)
        resultdict["VC_weight"] = visualcomplexity.get_visual_complexity_weight(pathToImg)
        resultdict["Symmetry_QTD"] = symmetry.get_symmetry(imageBW, minStd, minSize)
        resultdict["Colorfulness_HSV"] = colorfulness.colorfulness_hsv(imageColor)
        resultdict["Colorfulness_RGB"] = colorfulness.colorfulness_rgb(imageColor)
        resultdict["FacesCv2"] = facedetection.detect_faces_cv2(imageColor)
        resultdict["FacesFd"] = facedetection.detect_faces(pathToImg)
        resultdict["Number_of_Faces_Cv2"] = len(resultdict["FacesCv2"])
        resultdict["Number_of_Faces_fd"] = len(resultdict["FacesFd"])
        resultdict["Colors"] = colordetection.get_colors_w3c(imageColor, ncolors=140)
        A = spacebaseddecomposition.get_areas(imageColor_O)
        Adict = spacebaseddecomposition.text2image_ratio(A)
        resultdict["Number_of_Images"] = Adict["nImages"]
        resultdict["text2image_ratio"] = Adict["text2image_ratio"]
        resultdict["textArea"] = Adict["textArea"]
        resultdict["imageArea"] = Adict["imageArea"]
        resultdict["contrast_rms"] = contrast.contrast_rms(imageColor)
        resultdict["contrast_michelson"] = contrast.contrast_michelson(imageColor)
        resultdict["saturation"] = saturation.saturation(imageColor)
        resultdict["linesRatio"] = linesdetection.analyse_lines(imageColor)
        resultdict['selfSimilarity'] = selfsimilarity.get_self_similarity(img)

    # Return the result dictionary
    return resultdict

###############################################################################
#                                                                             #
#                                   DEBUG                                     #
#                                                                             #
###############################################################################

if __name__ == '__main__':
    basepath = os.path.dirname(os.path.realpath(__file__))

    # Path to a sample image for debugging   # Set the data path to use sample images
    data_folder = basepath + "/../share/data/"
    
    # Path to a sample image
    sample_img = data_folder + "panda.jpg"
    
    # Analyze the sample image using the 'complete' method
    results = analyze_image(sample_img, method='fast')
    # Print the analysis results
    print(results)

